@@ -8,23 +8,21 @@
 # copied, modified, or distributed except according to those terms.
 
 import base64
+import binascii
+import hashlib
 import json
 import struct
 import sys
 import tempfile
 import time
-import secrets
-import binascii
-import hashlib
+from typing import Optional
 
-from cryptography import x509
-from cryptography.hazmat.backends import default_backend
-from fido2.attestation import Attestation
+import secrets
 from fido2.client import Fido2Client
 from fido2.ctap import CtapError
-from fido2.ctap1 import CTAP1
-from fido2.ctap2 import CTAP2
-from fido2.hid import CTAPHID, CtapHidDevice
+from fido2.ctap1 import Ctap1
+from fido2.ctap2 import Ctap2
+from fido2.hid import CTAPHID, CtapHidDevice, open_device
 from intelhex import IntelHex
 
 import pynitrokey.exceptions
@@ -33,53 +31,69 @@ from pynitrokey.fido2.commands import SoloBootloader, SoloExtension
 
 
 class NKFido2Client:
-    def __init__(self,):
+    def __init__(
+        self,
+    ):
         self.origin = "https://example.org"
         self.host = "example.org"
         self.user_id = b"they"
         self.exchange = self.exchange_hid
         self.do_reboot = True
 
-    def use_u2f(self,):
+    def use_u2f(
+        self,
+    ):
         self.exchange = self.exchange_u2f
 
-    def use_hid(self,):
+    def use_hid(
+        self,
+    ):
         self.exchange = self.exchange_hid
 
     def set_reboot(self, val):
-        """ option to reboot after programming """
+        """option to reboot after programming"""
         self.do_reboot = val
 
-    def reboot(self,):
-        """ option to reboot after programming """
+    def reboot(
+        self,
+    ):
+        """option to reboot after programming"""
         try:
             self.exchange(SoloBootloader.reboot)
         except OSError:
             pass
 
-    def find_device(self, dev=None, solo_serial=None):
+    def find_device(self, dev=None, solo_serial: str = None):
+        devices = []
         if dev is None:
-            devices = list(CtapHidDevice.list_devices())
             if solo_serial is not None:
-                devices = [
-                    d for d in devices if d.descriptor["serial_number"] == solo_serial
-                ]
+                if solo_serial.startswith("device="):
+                    solo_serial = solo_serial.split("=")[1]
+                    dev = open_device(solo_serial)
+                else:
+                    devices = list(CtapHidDevice.list_devices())
+                    devices = [
+                        d for d in devices if d.descriptor.serial_number == solo_serial
+                    ]
+            else:
+                devices = list(CtapHidDevice.list_devices())
             if len(devices) > 1:
                 raise pynitrokey.exceptions.NonUniqueDeviceError
-            if len(devices) == 0:
-                raise RuntimeError("No FIDO device found")
-            dev = devices[0]
+            if len(devices) > 0:
+                dev = devices[0]
+        if dev is None:
+            raise RuntimeError("No FIDO device found")
         self.dev = dev
 
-        self.ctap1 = CTAP1(dev)
-        
+        self.ctap1 = Ctap1(dev)
+
         try:
-            self.ctap2 = CTAP2(dev)
+            self.ctap2: Optional[Ctap2] = Ctap2(dev)
         except CtapError as e:
             self.ctap2 = None
 
         try:
-            self.client = Fido2Client(dev, self.origin)
+            self.client: Optional[Fido2Client] = Fido2Client(dev, self.origin)
         except CtapError:
             print("Not using FIDO2 interface.")
             self.client = None
@@ -151,13 +165,17 @@ class NKFido2Client:
 
         return res.signature[1:]
 
-    def bootloader_version(self,):
+    def bootloader_version(
+        self,
+    ):
         data = self.exchange(SoloBootloader.version)
         if len(data) > 2:
             return (data[0], data[1], data[2])
         return (0, 0, data[0])
 
-    def solo_version(self,):
+    def solo_version(
+        self,
+    ):
         try:
             return self.send_data_hid(0x61, b"")
         except CtapError:
@@ -187,13 +205,18 @@ class NKFido2Client:
         """
         self.exchange(SoloBootloader.done, 0, sig)
 
-    def wink(self,):
+    def wink(
+        self,
+    ):
         self.send_data_hid(CTAPHID.WINK, b"")
 
-    def reset(self,):
+    def reset(
+        self,
+    ):
         self.ctap2.reset()
 
-    def make_credential(self, 
+    def make_credential(
+        self,
         host="nitrokeys.dev",
         user_id="they",
         serial=None,
@@ -201,7 +224,12 @@ class NKFido2Client:
         prompt="Touch your authenticator to generate a credential...",
         output=True,
         udp=False,
+        fingerprint_only=False,
     ):
+        """
+        fingerprint_only bool Return sha256 digest of the certificate, in a hex string format. Useful for detecting
+            device's model and firmware.
+        """
 
         user_id = user_id.encode()
         client = self.client
@@ -230,6 +258,14 @@ class NKFido2Client:
             pin=pin,
         ).attestation_object
 
+        if fingerprint_only:
+            if "x5c" not in attestation_object.att_statement:
+                raise ValueError("No x5c information available")
+            from hashlib import sha256
+
+            data = attestation_object.att_statement["x5c"]
+            return sha256(data[0]).digest().hex()
+
         credential = attestation_object.auth_data.credential_data
         credential_id = credential.credential_id
         if output:
@@ -237,7 +273,8 @@ class NKFido2Client:
 
         return credential_id
 
-    def simple_secret(self,
+    def simple_secret(
+        self,
         credential_id,
         secret_input,
         host="nitrokeys.dev",
@@ -286,14 +323,20 @@ class NKFido2Client:
 
         return output
 
-
     def cred_mgmt(self, pin):
-        client = self.get_current_fido_client()
-        token = client.client_pin.get_pin_token(pin)
-        ctap2 = CTAP2(self.get_current_hid_device())
-        return CredentialManagement(ctap2, client.client_pin.protocol, token)
+        # anyways unused code @todo
+        # client = self.get_current_fido_client()
+        dev = nkfido2.find(serial)
+        client = dev.client
+        client_pin = ClientPin(dev.ctap2)
+        client_pin.change_pin(old_pin, new_pin)
+        token = client_pin.get_pin_token(pin)
+        ctap2 = Ctap2(self.get_current_hid_device())
+        return CredentialManagement(ctap2, client_pin.protocol, token)
 
-    def enter_solo_bootloader(self,):
+    def enter_solo_bootloader(
+        self,
+    ):
         """
         If Nitrokey is configured as Nitrokey hacker or something similar,
         this command will tell the token to boot directly to the bootloader
@@ -317,7 +360,9 @@ class NKFido2Client:
             else:
                 raise (e)
 
-    def is_solo_bootloader(self,):
+    def is_solo_bootloader(
+        self,
+    ):
         try:
             self.bootloader_version()
             return True
@@ -328,11 +373,13 @@ class NKFido2Client:
                 raise (e)
         except Exception as e:
             # exception during bootloader version check, assume no bootloader
-            #local_print("could not get bootloader version")
+            # local_print("could not get bootloader version")
             pass
         return False
 
-    def enter_st_dfu(self,):
+    def enter_st_dfu(
+        self,
+    ):
         """
         If Nitrokey is configured as Nitrokey hacker or something similar,
         this command will tell the token to boot directly to the st DFU
@@ -346,7 +393,9 @@ class NKFido2Client:
         else:
             self.send_only_hid(SoloBootloader.HIDCommandEnterSTBoot, "")
 
-    def disable_solo_bootloader(self,):
+    def disable_solo_bootloader(
+        self,
+    ):
         """
         Disables the Nitrokey bootloader.  Only do this if you want to void the possibility
         of any updates.
@@ -367,8 +416,8 @@ class NKFido2Client:
             return base64.b64decode(helpers.from_websafe(f).encode())
 
         def isCorrectVersion(current, target):
-            """ current is tuple (x,y,z).  target is string '>=x.y.z'.
-                Return True if current satisfies the target expression.
+            """current is tuple (x,y,z).  target is string '>=x.y.z'.
+            Return True if current satisfies the target expression.
             """
             if "=" in target:
                 target = target.split("=")
@@ -406,7 +455,7 @@ class NKFido2Client:
                 #         print("using signature version", v)
                 #         sig = parseField(firmware_file_data["versions"][v]["signature"])
                 #         break
-                sig = parseField(firmware_file_data["versions"]['>2.5.3']["signature"])
+                sig = parseField(firmware_file_data["versions"][">2.5.3"]["signature"])
 
                 if sig is None:
                     raise RuntimeError(
@@ -440,7 +489,7 @@ class NKFido2Client:
         print("erasing firmware...")
         for i in range(seg[0], seg[1], chunk):
             s = i
-            e = min(i + chunk, seg[1])
+            e = min(i + chunk, seg[1])  # type: ignore
             data = ih.tobinarray(start=i, size=e - s)
             self.write_flash(i, data)
             total += chunk
@@ -457,7 +506,7 @@ class NKFido2Client:
         if self.do_reboot:
             try:
                 print("bootloader is verifying signature...")
-                print(f'Trying with {sig.hex()}')
+                print(f"Trying with {sig.hex()}")
                 self.verify_flash(sig)
                 print("...pass!")
                 success = True
